<?php

namespace Drupal\Tests\one_time_password\Functional;

use Drupal\simpletest\UserCreationTrait;
use Drupal\Tests\BrowserTestBase;

/**
 * Test the actual process of logging in with TFA.
 *
 * @coversDefaultClass \Drupal\one_time_password\UserLoginEnforce
 * @group one_time_password
 */
class UserLoginEnforceTest extends BrowserTestBase {

  use UserCreationTrait;

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = [
    'one_time_password',
  ];

  /**
   * Test the user login with TFA codes.
   */
  public function testUserLogin() {
    // Users without TFA enabled should be able to login as normal.
    $test_user = $this->createUser();
    $this->drupalLogin($test_user);
    $this->drupalLogout();

    // Create a user with TFA enabled.
    $test_user_with_tfa = $this->createUser();
    $test_user_with_tfa->one_time_password->regenerateOneTimePassword();
    $otp_object = $test_user_with_tfa->one_time_password->getOneTimePassword();
    $test_user_with_tfa->save();

    // Attempt to login with a user who has TFA enabled.
    $this->drupalGet('user/login');
    $this->submitForm([
      'name' => $test_user_with_tfa->getUsername(),
      'pass' => $test_user_with_tfa->passRaw,
    ], t('Log in'));
    $this->assertSession()->pageTextContains('The entered two factor authentication code is incorrect.');

    // Attempt to login with an incorrect TFA code.
    $this->drupalGet('user/login');
    $this->submitForm([
      'name' => $test_user_with_tfa->getUsername(),
      'pass' => $test_user_with_tfa->passRaw,
      'one_time_password' => '123456',
    ], t('Log in'));
    $this->assertSession()->pageTextContains('The entered two factor authentication code is incorrect.');

    // Attempt to login with the correct TFA code and ensure the user is
    // correctly logged in.
    $this->drupalGet('user/login');
    $this->submitForm([
      'name' => $test_user_with_tfa->getUsername(),
      'pass' => $test_user_with_tfa->passRaw,
      'one_time_password' => $otp_object->now(),
    ], t('Log in'));
    $test_user_with_tfa->sessionId = $this->getSession()->getCookie($this->getSessionName());
    $this->assertTrue($this->drupalUserIsLoggedIn($test_user_with_tfa), 'User was logged in successfully with a TFA code.');
  }

  /**
   * Test the wider login window works.
   */
  public function testLoginWindow() {
    $test_user_with_tfa = $this->createUser();
    $test_user_with_tfa->one_time_password->regenerateOneTimePassword();
    /** @var \OTPHP\TOTP $otp_object */
    $otp_object = $test_user_with_tfa->one_time_password->getOneTimePassword();
    $test_user_with_tfa->save();

    // Attempt to login with the code that would have been valid about three
    // minutes ago.
    $this->drupalGet('user/login');
    $this->submitForm([
      'name' => $test_user_with_tfa->getUsername(),
      'pass' => $test_user_with_tfa->passRaw,
      'one_time_password' => $otp_object->at(time() - 195),
    ], t('Log in'));

    $test_user_with_tfa->sessionId = $this->getSession()->getCookie($this->getSessionName());
    $this->assertTrue($this->drupalUserIsLoggedIn($test_user_with_tfa), 'User was logged in successfully with a TFA code.');
  }

  /**
   * Test no information disclosure through the flood table.
   */
  public function testFloodTableInformationDisclosure() {
    $this->config('user.flood')
      ->set('user_limit', 3)
      ->save();

    $tfa_user = $this->drupalCreateUser();
    $tfa_user->one_time_password->regenerateOneTimePassword();
    $tfa_user->save();

    // Login 5 times incorrectly to trigger a flood warning.
    foreach (range(0, 5) as $i) {
      $this->drupalPostForm('user/login', [
        'name' => $tfa_user->getUsername(),
        'pass' => 'incorrect password',
      ], 'Log in');
    }

    // Login with the correct password, which flood will deny, but ensure the
    // incorrect TFA code doesn't give us away.
    $this->drupalPostForm('user/login', [
      'name' => $tfa_user->getUsername(),
      'pass' => $tfa_user->passRaw,
      'one_time_password' => '123',
    ], 'Log in');
    $this->assertSession()->pageTextContains('There have been more than 3 failed login attempts');
    $this->assertSession()->pageTextNotContains('The entered two factor authentication code is incorrect.');
  }

}
